"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleError = exports.ErrorHandler = exports.notImplementedError = exports.unknownResourceError = exports.notAcceptableError = exports.authorizationError = exports.authenticationError = exports.dbConflictError = exports.badRequestError = void 0;
const celebrate_1 = require("celebrate");
const logger_1 = __importDefault(require("./logger"));
const badRequestError = (message) => {
    return new ErrorHandler(400, "Invalid Data", message);
};
exports.badRequestError = badRequestError;
const dbConflictError = (message) => {
    return new ErrorHandler(409, "InvalidData", message);
};
exports.dbConflictError = dbConflictError;
const authenticationError = (message) => {
    return new ErrorHandler(401, "Invalid Token", message);
};
exports.authenticationError = authenticationError;
const authorizationError = (message) => {
    return new ErrorHandler(403, "Insufficient permissions", message);
};
exports.authorizationError = authorizationError;
const notAcceptableError = (message) => {
    return new ErrorHandler(406, "Not Acceptable", message);
};
exports.notAcceptableError = notAcceptableError;
const unknownResourceError = (message) => {
    return new ErrorHandler(404, "UnknownResource", message);
};
exports.unknownResourceError = unknownResourceError;
const notImplementedError = (message) => {
    return new ErrorHandler(400, "NotImplementedError", message);
};
exports.notImplementedError = notImplementedError;
class ErrorHandler extends Error {
    constructor(statusCode, errorCode, message) {
        super(message);
        Object.setPrototypeOf(this, ErrorHandler.prototype);
        this.statusCode = statusCode;
        this.errorCode = errorCode;
        this.message = message;
    }
}
exports.ErrorHandler = ErrorHandler;
const handleKnownExceptions = (error, response) => {
    logger_1.default.error(Object.assign(Object.assign({}, error), { message: error.message }));
    const { statusCode, errorCode, message } = error;
    response
        .status(statusCode)
        .json({ errorMessage: `${errorCode} | ${message}` })
        .end();
};
const handleUnknownExceptions = (error, response) => {
    logger_1.default.error(Object.assign(Object.assign({}, error), { message: error.message }));
    return response
        .status(500)
        .json({ errorMessage: `${error.name} | ${error.message}` })
        .end();
};
const handleError = (error, response) => {
    if ((0, celebrate_1.isCelebrateError)(error)) {
        const message = formatCelebrateErrors(error);
        return response.status(400).json({ errorMessage: message });
    }
    error instanceof ErrorHandler
        ? handleKnownExceptions(error, response)
        : handleUnknownExceptions(error, response);
};
exports.handleError = handleError;
function formatCelebrateErrors(error) {
    let message = "";
    error.details.forEach((validationError) => {
        if (message) {
            message += "\n";
        }
        message += validationError.message;
    });
    return message;
}
//# sourceMappingURL=error.js.map